@@ -16,13 +16,13 @@ import (
 	"github.com/segmentio/ksuid"
 )
 
-func CompileForInternal(pctx *op.Context, p ast.Proc, r zio.Reader) (*Runtime, error) {
-	return CompileForInternalWithOrder(pctx, p, r, order.Layout{})
+func CompileForInternal(pctx *op.Context, o ast.Op, r zio.Reader) (*Runtime, error) {
+	return CompileForInternalWithOrder(pctx, o, r, order.Layout{})
 }
 
-func CompileForInternalWithOrder(pctx *op.Context, p ast.Proc, r zio.Reader, layout order.Layout) (*Runtime, error) {
+func CompileForInternalWithOrder(pctx *op.Context, o ast.Op, r zio.Reader, layout order.Layout) (*Runtime, error) {
 	adaptor := &internalAdaptor{}
-	runtime, err := New(pctx, p, adaptor, nil)
+	runtime, err := New(pctx, o, adaptor, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -49,10 +49,10 @@ func (*internalAdaptor) Layout(context.Context, dag.Source) order.Layout {
 	return order.Nil
 }
 
-func (*internalAdaptor) NewScheduler(context.Context, *zed.Context, dag.Source, extent.Span, zbuf.Filter, *dag.Filter) (op.Scheduler, error) {
+func (*internalAdaptor) NewScheduler(context.Context, *zed.Context, dag.Source, extent.Span, zbuf.Filter) (op.Scheduler, error) {
 	return nil, errors.New("invalid pool or file scan specified for internally streamed Zed query")
 }
 
-func (*internalAdaptor) Open(context.Context, *zed.Context, string, zbuf.Filter) (zbuf.Puller, error) {
+func (*internalAdaptor) Open(context.Context, *zed.Context, string, string, zbuf.Filter) (zbuf.Puller, error) {
 	return nil, errors.New("invalid file or URL access for internally streamed Zed query")
 }
