@@ -10,7 +10,6 @@ import (
 	"github.com/brimdata/zed"
 	"github.com/brimdata/zed/pkg/field"
 	"github.com/brimdata/zed/runtime/expr/coerce"
-	"github.com/brimdata/zed/runtime/expr/function"
 	"github.com/brimdata/zed/zcode"
 	"github.com/brimdata/zed/zson"
 )
@@ -19,6 +18,10 @@ type Evaluator interface {
 	Eval(Context, *zed.Value) *zed.Value
 }
 
+type Function interface {
+	Call(zed.Allocator, []zed.Value) *zed.Value
+}
+
 type Not struct {
 	zctx *zed.Context
 	expr Evaluator
@@ -72,7 +75,7 @@ func EvalBool(zctx *zed.Context, ectx Context, this *zed.Value, e Evaluator) (*z
 	if val.IsError() {
 		return val, false
 	}
-	return ectx.CopyValue(*zctx.NewErrorf("not type bool: %s", zson.MustFormatValue(*val))), false
+	return ectx.CopyValue(zctx.NewErrorf("not type bool: %s", zson.MustFormatValue(val))), false
 }
 
 func (a *And) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -357,7 +360,7 @@ func (c *Compare) Eval(ectx Context, this *zed.Value) *zed.Value {
 				result = 1
 			}
 		default:
-			return ectx.CopyValue(*c.zctx.NewErrorf("bad comparison type ID: %d", id))
+			return ectx.CopyValue(c.zctx.NewErrorf("bad comparison type ID: %d", id))
 		}
 	}
 	if c.convert(result) {
@@ -436,7 +439,7 @@ func (a *Add) Eval(ectx Context, this *zed.Value) *zed.Value {
 		// XXX GC
 		return ectx.NewValue(typ, zed.EncodeString(v1+v2))
 	}
-	return ectx.CopyValue(*a.zctx.NewErrorf("type %s incompatible with '+' operator", zson.FormatType(typ)))
+	return ectx.CopyValue(a.zctx.NewErrorf("type %s incompatible with '+' operator", zson.FormatType(typ)))
 }
 
 func (s *Subtract) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -459,7 +462,7 @@ func (s *Subtract) Eval(ectx Context, this *zed.Value) *zed.Value {
 		v1, v2 := s.operands.uints()
 		return ectx.NewValue(typ, zed.EncodeUint(v1-v2))
 	}
-	return ectx.CopyValue(*s.zctx.NewErrorf("type %s incompatible with '-' operator", zson.FormatType(typ)))
+	return ectx.CopyValue(s.zctx.NewErrorf("type %s incompatible with '-' operator", zson.FormatType(typ)))
 }
 
 func (m *Multiply) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -482,7 +485,7 @@ func (m *Multiply) Eval(ectx Context, this *zed.Value) *zed.Value {
 		v1, v2 := m.operands.uints()
 		return ectx.NewValue(typ, zed.EncodeUint(v1*v2))
 	}
-	return ectx.CopyValue(*m.zctx.NewErrorf("type %s incompatible with '*' operator", zson.FormatType(typ)))
+	return ectx.CopyValue(m.zctx.NewErrorf("type %s incompatible with '*' operator", zson.FormatType(typ)))
 }
 
 func (d *Divide) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -514,7 +517,7 @@ func (d *Divide) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		return ectx.NewValue(typ, zed.EncodeUint(v1/v2))
 	}
-	return ectx.CopyValue(*d.zctx.NewErrorf("type %s incompatible with '/' operator", zson.FormatType(typ)))
+	return ectx.CopyValue(d.zctx.NewErrorf("type %s incompatible with '/' operator", zson.FormatType(typ)))
 }
 
 func (m *Modulo) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -527,7 +530,7 @@ func (m *Modulo) Eval(ectx Context, this *zed.Value) *zed.Value {
 	}
 	typ := zed.LookupPrimitiveByID(id)
 	if zed.IsFloat(id) || !zed.IsNumber(id) {
-		return ectx.CopyValue(*m.zctx.NewErrorf("type %s incompatible with '%%' operator", zson.FormatType(typ)))
+		return ectx.CopyValue(m.zctx.NewErrorf("type %s incompatible with '%%' operator", zson.FormatType(typ)))
 	}
 	if zed.IsSigned(id) {
 		x, y := m.operands.ints()
@@ -575,7 +578,7 @@ func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		v := zed.DecodeInt(val.Bytes)
 		if v == math.MinInt8 {
-			return ectx.CopyValue(*u.zctx.NewErrorf("unary '-' underflow: int8(%d)", v))
+			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' underflow: int8(%d)", v))
 		}
 		return ectx.NewValue(typ, zed.EncodeInt(-v))
 	case zed.IDInt16:
@@ -584,7 +587,7 @@ func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		v := zed.DecodeInt(val.Bytes)
 		if v == math.MinInt16 {
-			return ectx.CopyValue(*u.zctx.NewErrorf("unary '-' underflow: int16(%d)", v))
+			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' underflow: int16(%d)", v))
 		}
 		return ectx.NewValue(typ, zed.EncodeInt(-v))
 	case zed.IDInt32:
@@ -593,7 +596,7 @@ func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		v := zed.DecodeInt(val.Bytes)
 		if v == math.MinInt32 {
-			return ectx.CopyValue(*u.zctx.NewErrorf("unary '-' underflow: int32(%d)", v))
+			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' underflow: int32(%d)", v))
 		}
 		return ectx.NewValue(typ, zed.EncodeInt(-v))
 	case zed.IDInt64:
@@ -602,7 +605,7 @@ func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		v := zed.DecodeInt(val.Bytes)
 		if v == math.MinInt64 {
-			return ectx.CopyValue(*u.zctx.NewErrorf("unary '-' underflow: int64(%d)", v))
+			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' underflow: int64(%d)", v))
 		}
 		return ectx.NewValue(typ, zed.EncodeInt(-v))
 	case zed.IDUint8:
@@ -611,7 +614,7 @@ func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		v := zed.DecodeUint(val.Bytes)
 		if v > math.MaxInt8 {
-			return ectx.CopyValue(*u.zctx.NewErrorf("unary '-' overflow: uint8(%d)", v))
+			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' overflow: uint8(%d)", v))
 		}
 		return ectx.NewValue(zed.TypeInt8, zed.EncodeInt(-int64(v)))
 	case zed.IDUint16:
@@ -620,7 +623,7 @@ func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		v := zed.DecodeUint(val.Bytes)
 		if v > math.MaxInt16 {
-			return ectx.CopyValue(*u.zctx.NewErrorf("unary '-' overflow: uint16(%d)", v))
+			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' overflow: uint16(%d)", v))
 		}
 		return ectx.NewValue(zed.TypeInt16, zed.EncodeInt(-int64(v)))
 	case zed.IDUint32:
@@ -629,7 +632,7 @@ func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		v := zed.DecodeUint(val.Bytes)
 		if v > math.MaxInt32 {
-			return ectx.CopyValue(*u.zctx.NewErrorf("unary '-' overflow: uint32(%d)", v))
+			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' overflow: uint32(%d)", v))
 		}
 		return ectx.NewValue(zed.TypeInt32, zed.EncodeInt(-int64(v)))
 	case zed.IDUint64:
@@ -638,7 +641,7 @@ func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 		}
 		v := zed.DecodeUint(val.Bytes)
 		if v > math.MaxInt64 {
-			return ectx.CopyValue(*u.zctx.NewErrorf("unary '-' overflow: uint64(%d)", v))
+			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' overflow: uint64(%d)", v))
 		}
 		return ectx.NewValue(zed.TypeInt64, zed.EncodeInt(-int64(v)))
 	}
@@ -706,7 +709,7 @@ func (i *Index) Eval(ectx Context, this *zed.Value) *zed.Value {
 func indexArray(zctx *zed.Context, ectx Context, typ *zed.TypeArray, array zcode.Bytes, index *zed.Value) *zed.Value {
 	id := index.Type.ID()
 	if !zed.IsInteger(id) {
-		return ectx.CopyValue(*zctx.NewErrorf("array index is not an integer"))
+		return ectx.CopyValue(zctx.NewErrorf("array index is not an integer"))
 	}
 	var idx int
 	if zed.IsSigned(id) {
@@ -724,14 +727,14 @@ func indexArray(zctx *zed.Context, ectx Context, typ *zed.TypeArray, array zcode
 func indexRecord(zctx *zed.Context, ectx Context, typ *zed.TypeRecord, record zcode.Bytes, index *zed.Value) *zed.Value {
 	id := index.Type.ID()
 	if id != zed.IDString {
-		return ectx.CopyValue(*zctx.NewErrorf("record index is not a string"))
+		return ectx.CopyValue(zctx.NewErrorf("record index is not a string"))
 	}
 	field := zed.DecodeString(index.Bytes)
 	val := zed.NewValue(typ, record).Deref(field)
 	if val == nil {
 		return zctx.Missing()
 	}
-	return ectx.CopyValue(*val)
+	return ectx.CopyValue(val)
 }
 
 func indexMap(zctx *zed.Context, ectx Context, typ *zed.TypeMap, mapBytes zcode.Bytes, key *zed.Value) *zed.Value {
@@ -785,12 +788,12 @@ func (c *Conditional) Eval(ectx Context, this *zed.Value) *zed.Value {
 
 type Call struct {
 	zctx  *zed.Context
-	fn    function.Interface
+	fn    Function
 	exprs []Evaluator
 	args  []zed.Value
 }
 
-func NewCall(zctx *zed.Context, fn function.Interface, exprs []Evaluator) *Call {
+func NewCall(zctx *zed.Context, fn Function, exprs []Evaluator) *Call {
 	return &Call{
 		zctx:  zctx,
 		fn:    fn,
@@ -806,45 +809,6 @@ func (c *Call) Eval(ectx Context, this *zed.Value) *zed.Value {
 	return c.fn.Call(ectx, c.args)
 }
 
-// https://github.com/brimdata/zed/blob/main/docs/language/functions.md#has
-type Has struct {
-	exprs []Evaluator
-}
-
-func NewHas(exprs []Evaluator) *Has {
-	return &Has{exprs}
-}
-
-func (h *Has) Eval(ectx Context, this *zed.Value) *zed.Value {
-	for _, e := range h.exprs {
-		val := e.Eval(ectx, this)
-		if val.IsError() {
-			if val.IsMissing() || val.IsQuiet() {
-				return zed.False
-			}
-			return val
-		}
-	}
-	return zed.True
-}
-
-// https://github.com/brimdata/zed/blob/main/docs/language/functions.md#missing
-type Missing struct {
-	has *Has
-}
-
-func NewMissing(exprs []Evaluator) *Missing {
-	return &Missing{NewHas(exprs)}
-}
-
-func (m *Missing) Eval(ectx Context, this *zed.Value) *zed.Value {
-	val := m.has.Eval(ectx, this)
-	if val.Type == zed.TypeBool {
-		val = zed.Not(val.Bytes)
-	}
-	return val
-}
-
 func NewCast(zctx *zed.Context, expr Evaluator, typ zed.Type) (Evaluator, error) {
 	// XXX should handle named type casts. need type context.
 	// compile is going to need a local type context to create literals
@@ -877,3 +841,16 @@ type Assignment struct {
 	LHS field.Path
 	RHS Evaluator
 }
+
+func NewAssignments(zctx *zed.Context, dsts field.List, srcs field.List) (field.List, []Evaluator) {
+	if len(srcs) != len(dsts) {
+		panic("NewAssignments: argument mismatch")
+	}
+	var resolvers []Evaluator
+	var fields field.List
+	for k, dst := range dsts {
+		fields = append(fields, dst)
+		resolvers = append(resolvers, NewDottedExpr(zctx, srcs[k]))
+	}
+	return fields, resolvers
+}
