@@ -20,6 +20,7 @@ import (
 	"github.com/brimdata/zed/service/srverr"
 	"github.com/brimdata/zed/zio"
 	"github.com/brimdata/zed/zio/anyio"
+	"github.com/segmentio/ksuid"
 )
 
 func handleQuery(c *Core, w *ResponseWriter, r *Request) {
@@ -37,12 +38,7 @@ func handleQuery(c *Core, w *ResponseWriter, r *Request) {
 	// The client must look at the return code and interpret the result
 	// accordingly and when it sees a ZNG error after underway,
 	// the error should be relay that to the caller/user.
-	query, err := compiler.ParseProc(req.Query)
-	if err != nil {
-		w.Error(srverr.ErrInvalid(err))
-		return
-	}
-	format, err := api.MediaTypeToFormat(r.Header.Get("Accept"), DefaultZedFormat)
+	query, err := compiler.ParseOp(req.Query)
 	if err != nil {
 		w.Error(srverr.ErrInvalid(err))
 		return
@@ -53,7 +49,7 @@ func handleQuery(c *Core, w *ResponseWriter, r *Request) {
 		return
 	}
 	flusher, _ := w.ResponseWriter.(http.Flusher)
-	writer, err := queryio.NewWriter(zio.NopCloser(w), format, flusher)
+	writer, err := queryio.NewWriter(zio.NopCloser(w), w.Format, flusher)
 	if err != nil {
 		w.Error(err)
 		return
@@ -408,12 +404,30 @@ func handleDelete(c *Core, w *ResponseWriter, r *Request) {
 		w.Error(err)
 		return
 	}
-	ids, err := branch.LookupTags(r.Context(), payload.ObjectIDs)
-	if err != nil {
-		w.Error(err)
-		return
+	var commit ksuid.KSUID
+	if len(payload.ObjectIDs) > 0 {
+		if payload.Where != "" {
+			w.Error(srverr.ErrInvalid("object_ids and where cannot both be set"))
+			return
+		}
+		tags, err := lakeparse.ParseIDs(payload.ObjectIDs)
+		if err != nil {
+			w.Error(srverr.ErrInvalid(err))
+			return
+		}
+		ids, err := branch.LookupTags(r.Context(), tags)
+		if err != nil {
+			w.Error(err)
+			return
+		}
+		commit, err = branch.Delete(r.Context(), ids, message.Author, message.Body)
+	} else {
+		if payload.Where == "" {
+			w.Error(srverr.ErrInvalid("either object_ids or where must be set"))
+			return
+		}
+		commit, err = branch.DeleteByPredicate(r.Context(), c.root, payload.Where, message.Author, message.Body, message.Meta)
 	}
-	commit, err := branch.Delete(r.Context(), ids, message.Author, message.Body)
 	if err != nil {
 		w.Error(err)
 		return
@@ -455,7 +469,12 @@ func handleIndexApply(c *Core, w *ResponseWriter, r *Request, branch *lake.Branc
 	if !r.Unmarshal(w, &req) {
 		return
 	}
-	tags, err := branch.LookupTags(r.Context(), req.Tags)
+	ss, err := lakeparse.ParseIDs(req.Tags)
+	if err != nil {
+		w.Error(srverr.ErrInvalid(err))
+		return
+	}
+	tags, err := branch.LookupTags(r.Context(), ss)
 	if err != nil {
 		w.Error(err)
 		return
